@@ -47,17 +47,27 @@ impl FromStr for SelfSigning {
     type Err = Error;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        match &s[..1] {
+        match s
+            .get(..1)
+            .ok_or_else(|| Error::DeserializeError("Empty prefix".into()))?
+        {
             "0" => match &s[1..2] {
                 "B" => Ok(Self::Ed25519Sha512),
                 "C" => Ok(Self::ECDSAsecp256k1Sha256),
-                _ => Err(Error::DeserializeError("Unknown signature type code".into())),
+                _ => Err(Error::DeserializeError(
+                    "Unknown signature type code".into(),
+                )),
             },
             "1" => match &s[1..4] {
                 "AAE" => Ok(Self::Ed448),
-                _ => Err(Error::DeserializeError("Unknown signature type code".into())),
+                _ => Err(Error::DeserializeError(
+                    "Unknown signature type code".into(),
+                )),
             },
-            _ => Err(Error::DeserializeError(format!("Unknown master code: {}", s))),
+            _ => Err(Error::DeserializeError(format!(
+                "Unknown master code: {}",
+                s
+            ))),
         }
     }
 }
