@@ -101,4 +101,14 @@ pub enum Error {
 
     #[error("mutex is poisoned")]
     MutexPoisoned,
+
+    #[error("Incorrect event digest")]
+    IncorrectDigest,
+
+    #[cfg(feature = "query")]
+    #[error(transparent)]
+    QueryError(#[from] crate::query::QueryError),
+
+    #[error("Public Key Error: {0}")]
+    PublicKeyError(String),
 }
