@@ -5,13 +5,14 @@ use crate::{
     event::{
         event_data::{
             delegated::DelegatedInceptionEvent, interaction::InteractionEvent,
-            rotation::RotationEvent, Receipt,
+            rotation::RotationEvent,
         },
         sections::{threshold::SignatureThreshold, WitnessConfig},
         SerializationFormats,
     },
     event::{
         event_data::{inception::InceptionEvent, EventData},
+        receipt::Receipt,
         sections::seal::Seal,
         sections::InceptionWitnessConfig,
         sections::KeyConfig,
@@ -23,8 +24,10 @@ use crate::{
 use ed25519_dalek::Keypair;
 use rand::rngs::OsRng;
 
+use super::{EventTypeTag, KeyEvent};
+
 pub struct EventMsgBuilder {
-    event_type: EventType,
+    event_type: EventTypeTag,
     prefix: IdentifierPrefix,
     sn: u64,
     key_threshold: SignatureThreshold,
@@ -34,33 +37,16 @@ pub struct EventMsgBuilder {
     prev_event: SelfAddressingPrefix,
     data: Vec<Seal>,
     delegator: IdentifierPrefix,
+    witness_threshold: u64,
+    witnesses: Vec<BasicPrefix>,
+    witness_to_add: Vec<BasicPrefix>,
+    witness_to_remove: Vec<BasicPrefix>,
     format: SerializationFormats,
     derivation: SelfAddressing,
 }
 
-#[derive(Clone, Debug)]
-pub enum EventType {
-    Inception,
-    Rotation,
-    Interaction,
-    DelegatedInception,
-    DelegatedRotation,
-}
-
-impl EventType {
-    pub fn is_establishment_event(&self) -> bool {
-        match self {
-            EventType::Inception
-            | EventType::Rotation
-            | EventType::DelegatedInception
-            | EventType::DelegatedRotation => true,
-            _ => false,
-        }
-    }
-}
-
 impl EventMsgBuilder {
-    pub fn new(event_type: EventType) -> Self {
+    pub fn new(event_type: EventTypeTag) -> Self {
         let mut rng = OsRng {};
         let kp = Keypair::generate(&mut rng);
         let nkp = Keypair::generate(&mut rng);
@@ -78,6 +64,10 @@ impl EventMsgBuilder {
             prev_event: SelfAddressing::Blake3_256.derive(&[0u8; 32]),
             data: vec![],
             delegator: IdentifierPrefix::default(),
+            witness_threshold: 0,
+            witnesses: vec![],
+            witness_to_add: vec![],
+            witness_to_remove: vec![],
             format: SerializationFormats::JSON,
             derivation: SelfAddressing::Blake3_256,
         }
@@ -134,38 +124,58 @@ impl EventMsgBuilder {
         }
     }
 
-    pub fn build(self) -> Result<EventMessage, Error> {
+    pub fn with_witness_list(self, witnesses: &[BasicPrefix]) -> Self {
+        EventMsgBuilder {
+            witnesses: witnesses.to_vec(),
+            ..self
+        }
+    }
+
+    pub fn with_witness_to_add(self, witness_to_add: &[BasicPrefix]) -> Self {
+        EventMsgBuilder {
+            witness_to_add: witness_to_add.to_vec(),
+            ..self
+        }
+    }
+
+    pub fn with_witness_to_remove(self, witness_to_remove: &[BasicPrefix]) -> Self {
+        EventMsgBuilder {
+            witness_to_remove: witness_to_remove.to_vec(),
+            ..self
+        }
+    }
+
+    pub fn build(self) -> Result<EventMessage<KeyEvent>, Error> {
         let next_key_hash =
             nxt_commitment(&self.next_key_threshold, &self.next_keys, &self.derivation);
         let key_config = KeyConfig::new(self.keys, Some(next_key_hash), Some(self.key_threshold));
         let prefix = if self.prefix == IdentifierPrefix::default() {
             if key_config.public_keys.len() == 1 {
-                IdentifierPrefix::Basic(key_config.clone().public_keys[0].clone())
+                IdentifierPrefix::Basic(key_config.public_keys[0].clone())
             } else {
                 let icp_data = InceptionEvent::new(key_config.clone(), None, None)
                     .incept_self_addressing(self.derivation.clone(), self.format)?;
-                icp_data.event.prefix
+                icp_data.event.get_prefix()
             }
         } else {
             self.prefix
         };
 
         Ok(match self.event_type {
-            EventType::Inception => {
+            EventTypeTag::Icp => {
                 let icp_event = InceptionEvent {
                     key_config,
-                    witness_config: InceptionWitnessConfig::default(),
+                    witness_config: InceptionWitnessConfig {
+                        tally: self.witness_threshold,
+                        initial_witnesses: self.witnesses,
+                    },
                     inception_configuration: vec![],
                     data: vec![],
                 };
 
                 match prefix {
-                    IdentifierPrefix::Basic(_) => Event {
-                        prefix,
-                        sn: 0,
-                        event_data: EventData::Icp(icp_event),
-                    }
-                    .to_message(self.format)?,
+                    IdentifierPrefix::Basic(_) => Event::new(prefix, 0, EventData::Icp(icp_event))
+                        .to_message(self.format, &self.derivation)?,
                     IdentifierPrefix::SelfAddressing(_) => {
                         icp_event.incept_self_addressing(self.derivation, self.format)?
                     }
@@ -173,27 +183,31 @@ impl EventMsgBuilder {
                 }
             }
 
-            EventType::Rotation => Event {
+            EventTypeTag::Rot => Event::new(
                 prefix,
-                sn: self.sn,
-                event_data: EventData::Rot(RotationEvent {
+                self.sn,
+                EventData::Rot(RotationEvent {
                     previous_event_hash: self.prev_event,
                     key_config,
-                    witness_config: WitnessConfig::default(),
+                    witness_config: WitnessConfig {
+                        tally: self.witness_threshold,
+                        prune: self.witness_to_remove,
+                        graft: self.witness_to_add,
+                    },
                     data: self.data,
                 }),
-            }
-            .to_message(self.format)?,
-            EventType::Interaction => Event {
+            )
+            .to_message(self.format, &self.derivation)?,
+            EventTypeTag::Ixn => Event::new(
                 prefix,
-                sn: self.sn,
-                event_data: EventData::Ixn(InteractionEvent {
+                self.sn,
+                EventData::Ixn(InteractionEvent {
                     previous_event_hash: self.prev_event,
                     data: self.data,
                 }),
-            }
-            .to_message(self.format)?,
-            EventType::DelegatedInception => {
+            )
+            .to_message(self.format, &self.derivation)?,
+            EventTypeTag::Dip => {
                 let icp_data = InceptionEvent {
                     key_config,
                     witness_config: InceptionWitnessConfig::default(),
@@ -206,20 +220,17 @@ impl EventMsgBuilder {
                 }
                 .incept_self_addressing(self.derivation, self.format)?
             }
-            EventType::DelegatedRotation => {
+            EventTypeTag::Drt => {
                 let rotation_data = RotationEvent {
                     previous_event_hash: self.prev_event,
                     key_config,
                     witness_config: WitnessConfig::default(),
                     data: self.data,
                 };
-                Event {
-                    prefix,
-                    sn: self.sn,
-                    event_data: EventData::Drt(rotation_data),
-                }
-                .to_message(self.format)?
+                Event::new(prefix, self.sn, EventData::Drt(rotation_data))
+                    .to_message(self.format, &self.derivation)?
             }
+            _ => return Err(Error::SemanticError("Not key event".into())),
         })
     }
 }
@@ -227,19 +238,21 @@ impl EventMsgBuilder {
 pub struct ReceiptBuilder {
     format: SerializationFormats,
     derivation: SelfAddressing,
-    receipted_event: EventMessage,
+    receipted_event: EventMessage<KeyEvent>,
 }
 
-impl ReceiptBuilder {
-    pub fn new() -> Self {
-        let default_event = EventMessage::default();
+impl Default for ReceiptBuilder {
+    fn default() -> Self {
+        let default_event = EventMsgBuilder::new(EventTypeTag::Icp).build().unwrap();
         Self {
             format: SerializationFormats::JSON,
             derivation: SelfAddressing::Blake3_256,
             receipted_event: default_event,
         }
     }
+}
 
+impl ReceiptBuilder {
     pub fn with_format(self, format: SerializationFormats) -> Self {
         Self { format, ..self }
     }
@@ -248,20 +261,21 @@ impl ReceiptBuilder {
         Self { derivation, ..self }
     }
 
-    pub fn with_receipted_event(self, receipted_event: EventMessage) -> Self {
+    pub fn with_receipted_event(self, receipted_event: EventMessage<KeyEvent>) -> Self {
         Self {
             receipted_event,
             ..self
         }
     }
 
-    pub fn build(&self) -> Result<EventMessage, Error> {
-        Event {
-            prefix: self.receipted_event.event.prefix.clone(),
-            sn: self.receipted_event.event.sn,
-            event_data: EventData::Rct(Receipt {
-                receipted_event_digest: self.derivation.derive(&self.receipted_event.serialize()?),
-            }),
+    pub fn build(&self) -> Result<EventMessage<Receipt>, Error> {
+        let prefix = self.receipted_event.event.get_prefix();
+        let sn = self.receipted_event.event.get_sn();
+        let receipted_event_digest = self.derivation.derive(&self.receipted_event.serialize()?);
+        Receipt {
+            receipted_event_digest,
+            sn,
+            prefix,
         }
         .to_message(self.format)
     }
@@ -269,8 +283,8 @@ impl ReceiptBuilder {
 
 #[test]
 fn test_multisig_prefix_derivation() {
-    // Keys taken from keripy: keripy/tests/core/test_eventing.py::2405-2406
-    let expected_event = br#"{"v":"KERI10JSON00014b_","i":"EsiHneigxgDopAidk_dmHuiUJR3kAaeqpgOAj9ZZd4q8","s":"0","t":"icp","kt":"2","k":["DSuhyBcPZEZLK-fcw5tzHn2N46wRCG_ZOoeKtWTOunRA","DVcuJOOJF1IE8svqEtrSuyQjGTd2HhfAkt9y2QkUtFJI","DT1iAhBWCkvChxNWsby2J0pJyxBIxbAtbLA0Ljx-Grh8"],"n":"E9izzBkXX76sqt0N-tfLzJeRqj0W56p4pDQ_ZqNCDpyw","bt":"0","b":[],"c":[],"a":[]}"#;
+    // Keys taken from keripy: keripy/tests/core/test_eventing.py::test_multisig_digprefix (line 2255)
+    let expected_event = br#"{"v":"KERI10JSON00017e_","t":"icp","d":"ELYk-z-SuTIeDncLr6GhwVUKnv3n3F1bF18qkXNd2bpk","i":"ELYk-z-SuTIeDncLr6GhwVUKnv3n3F1bF18qkXNd2bpk","s":"0","kt":"2","k":["DSuhyBcPZEZLK-fcw5tzHn2N46wRCG_ZOoeKtWTOunRA","DVcuJOOJF1IE8svqEtrSuyQjGTd2HhfAkt9y2QkUtFJI","DT1iAhBWCkvChxNWsby2J0pJyxBIxbAtbLA0Ljx-Grh8"],"n":"E9izzBkXX76sqt0N-tfLzJeRqj0W56p4pDQ_ZqNCDpyw","bt":"0","b":[],"c":[],"a":[]}"#;
     let keys: Vec<BasicPrefix> = vec![
         "DSuhyBcPZEZLK-fcw5tzHn2N46wRCG_ZOoeKtWTOunRA"
             .parse()
@@ -294,7 +308,7 @@ fn test_multisig_prefix_derivation() {
             .unwrap(),
     ];
 
-    let msg_builder = EventMsgBuilder::new(EventType::Inception)
+    let msg_builder = EventMsgBuilder::new(EventTypeTag::Icp)
         .with_keys(keys)
         .with_next_keys(next_keys)
         .with_threshold(&SignatureThreshold::Simple(2))
