@@ -1,5 +1,13 @@
-use super::event_msg_builder::{EventMsgBuilder, EventType};
-use crate::{derivation::{basic::Basic, self_addressing::SelfAddressing, self_signing::SelfSigning}, error::Error, event::sections::{key_config::nxt_commitment, threshold::SignatureThreshold}, keys::{PrivateKey, PublicKey}, prefix::{AttachedSignaturePrefix, BasicPrefix, IdentifierPrefix, SelfAddressingPrefix }, state::IdentifierState};
+use super::event_msg_builder::EventMsgBuilder;
+use crate::{
+    derivation::{basic::Basic, self_addressing::SelfAddressing, self_signing::SelfSigning},
+    error::Error,
+    event::sections::{key_config::nxt_commitment, threshold::SignatureThreshold},
+    event_message::EventTypeTag,
+    keys::{PrivateKey, PublicKey},
+    prefix::{AttachedSignaturePrefix, BasicPrefix, IdentifierPrefix, SelfAddressingPrefix},
+    state::IdentifierState,
+};
 use ed25519_dalek::Keypair;
 use rand::rngs::OsRng;
 
@@ -38,7 +46,7 @@ fn get_initial_test_data() -> Result<TestStateData, Error> {
 /// Construct mock event message from `event_type` and `state_data`, apply it to
 /// `IdentifierState` in `state_data.state` and check if it was updated correctly.
 fn test_update_identifier_state(
-    event_type: EventType,
+    event_type: EventTypeTag,
     state_data: TestStateData,
 ) -> Result<TestStateData, Error> {
     // Get current and next key_pairs from argument.
@@ -72,7 +80,7 @@ fn test_update_identifier_state(
         .with_keys(vec![current_key_pref.clone()])
         .with_next_keys(vec![next_key_prefix])
         .build()?;
-    let prefix = event_msg.event.prefix.clone();
+    let prefix = event_msg.event.get_prefix();
 
     // Serialize event message before signing.
     let sed = event_msg.serialize()?;
@@ -85,7 +93,7 @@ fn test_update_identifier_state(
     };
 
     // Attach sign to event message.
-    let signed_event = event_msg.sign( vec![attached_sig.clone()], None);
+    let signed_event = event_msg.sign(vec![attached_sig.clone()], None);
 
     // Apply event to current IdentifierState.
     let new_state = state_data.state.apply(&signed_event)?;
@@ -105,16 +113,15 @@ fn test_update_identifier_state(
     };
 
     // Check if state is updated correctly.
-    assert_eq!(new_state.prefix, prefix.clone());
+    assert_eq!(new_state.prefix, prefix);
     assert_eq!(new_state.sn, state_data.sn);
-    assert_eq!(new_state.last, sed);
+    assert_eq!(new_state.last_event_digest, event_msg.get_digest());
     assert_eq!(new_state.current.public_keys.len(), 1);
     assert_eq!(new_state.current.public_keys[0], current_key_pref);
     assert_eq!(new_state.current.threshold, SignatureThreshold::Simple(1));
     assert_eq!(new_state.current.threshold_key_digest, Some(next_dig));
     assert_eq!(new_state.witnesses, vec![]);
     assert_eq!(new_state.tally, 0);
-    assert_eq!(new_state.delegates, vec![]);
 
     let mut new_history = state_data.keys_history.clone();
     // If event_type is establishment event, append current key to keys
@@ -123,13 +130,13 @@ fn test_update_identifier_state(
         new_history.push(current_key_pref);
     }
     // Current event will be previous event for the next one, so return its hash.
-    let prev_event_hash = SelfAddressing::Blake3_256.derive(&sed);
+    let prev_event_hash = event_msg.get_digest();
     // Compute sn for next event.
     let next_sn = state_data.sn + 1;
 
     Ok(TestStateData {
         state: new_state,
-        prefix: prefix,
+        prefix,
         keys_history: new_history,
         prev_event_hash,
         sn: next_sn,
@@ -140,7 +147,7 @@ fn test_update_identifier_state(
 
 /// For given sequence of EventTypes check wheather `IdentifierState` is updated correctly
 /// by applying `test_update_identifier_state` sequentially.
-pub fn test_mock_event_sequence(sequence: Vec<EventType>) -> Result<TestStateData, Error> {
+pub fn test_mock_event_sequence(sequence: Vec<EventTypeTag>) -> Result<TestStateData, Error> {
     let mut st = get_initial_test_data();
 
     let step = |event_type, state_data: Result<TestStateData, Error>| {
