@@ -1,34 +1,43 @@
-use std::{convert::TryFrom, sync::{Arc, Mutex}};
+use std::{
+    convert::TryFrom,
+    sync::{Arc, Mutex},
+};
 
-use crate::{database::sled::SledEventDatabase, derivation::basic::Basic, derivation::self_addressing::SelfAddressing, derivation::self_signing::SelfSigning, error::Error, event::sections::seal::{
-        DigestSeal,
-        Seal
-    }, event::{
-        event_data::EventData,
-        Event,
-        EventMessage,
-        SerializationFormats
-    }, event::{
-        event_data::{
-            Receipt,
-            InteractionEvent,
+use crate::{
+    database::sled::SledEventDatabase,
+    derivation::basic::Basic,
+    derivation::self_addressing::SelfAddressing,
+    derivation::self_signing::SelfSigning,
+    error::Error,
+    event::sections::seal::{DigestSeal, Seal},
+    event::{event_data::EventData, receipt::Receipt, Event, EventMessage, SerializationFormats},
+    event::{event_data::InteractionEvent, sections::seal::EventSeal},
+    event_message::event_msg_builder::EventMsgBuilder,
+    event_message::{
+        key_event_message::KeyEvent,
+        signed_event_message::{
+            Message, SignedEventMessage, SignedNontransferableReceipt, SignedTransferableReceipt,
         },
-        sections::seal::EventSeal
-    }, event_message::{signed_event_message::{SignedEventMessage, SignedNontransferableReceipt, SignedTransferableReceipt, Message}}, event_message::{
-        event_msg_builder::{EventMsgBuilder, EventType},
-    }, event_parsing::{SignedEventData, message::{signed_event_stream, signed_message}}, keys::PublicKey, prefix::AttachedSignaturePrefix, prefix::{
-        BasicPrefix,
-        IdentifierPrefix,
-        SelfSigningPrefix
-    }, processor::EventProcessor, signer::KeyManager, state::{
-        EventSemantics,
-        IdentifierState
-    }};
+        EventTypeTag,
+    },
+    event_parsing::{
+        message::{signed_event_stream, signed_message},
+        SignedEventData,
+    },
+    keys::PublicKey,
+    prefix::AttachedSignaturePrefix,
+    prefix::{BasicPrefix, IdentifierPrefix, SelfSigningPrefix},
+    processor::EventProcessor,
+    signer::KeyManager,
+    state::{EventSemantics, IdentifierState},
+};
 #[cfg(feature = "wallet")]
 use universal_wallet::prelude::{Content, UnlockedWallet};
 
 #[cfg(test)]
 mod test;
+#[cfg(feature = "query")]
+pub mod witness;
 pub struct Keri<K: KeyManager + 'static> {
     prefix: IdentifierPrefix,
     key_manager: Arc<Mutex<K>>,
@@ -106,24 +115,31 @@ impl<K: KeyManager> Keri<K> {
         }
     }
 
-    pub fn incept(&mut self) -> Result<SignedEventMessage, Error> {
+    pub fn incept(
+        &mut self,
+        initial_witness: Option<Vec<BasicPrefix>>,
+    ) -> Result<SignedEventMessage, Error> {
         let km = self.key_manager.lock().map_err(|_| Error::MutexPoisoned)?;
-        let icp = EventMsgBuilder::new(EventType::Inception)
+        let icp = EventMsgBuilder::new(EventTypeTag::Icp)
             .with_prefix(&self.prefix)
-            .with_keys(vec![Basic::Ed25519.derive(km.public_key())])
-            .with_next_keys(vec![Basic::Ed25519.derive(km.next_public_key())])
+            .with_keys(vec![Basic::Ed25519.derive(km.public_key()?)])
+            .with_next_keys(vec![Basic::Ed25519.derive(km.next_public_key()?)])
+            .with_witness_list(&initial_witness.unwrap_or_default())
             .build()?;
 
-        let signed = icp.sign(vec![AttachedSignaturePrefix::new(
-            SelfSigning::Ed25519Sha512,
-            km.sign(&icp.serialize()?)?,
-            0,
-        )], None);
+        let signed = icp.sign(
+            vec![AttachedSignaturePrefix::new(
+                SelfSigning::Ed25519Sha512,
+                km.sign(&icp.serialize()?)?,
+                0,
+            )],
+            None,
+        );
 
         self.processor
-            .process(Message::Event(signed.clone()))?;
+            .process(Message::Event(Box::new(signed.clone())))?;
 
-        self.prefix = icp.event.prefix;
+        self.prefix = icp.event.get_prefix();
 
         Ok(signed)
     }
@@ -138,31 +154,36 @@ impl<K: KeyManager> Keri<K> {
     ///  where `SignedEventMessage` is ICP event including all provided keys + directly fetched
     ///  verification key, signed with it's private key via KeyManager and serialized.
     ///
-    pub fn incept_with_extra_keys(&mut self, extra_keys: impl IntoIterator<Item = (Basic, PublicKey)>) 
-        -> Result<SignedEventMessage, Error> {
-            let mut keys: Vec<BasicPrefix> = extra_keys
-                .into_iter()
-                .map(|(key_type, key)| key_type.derive(key)).collect();
-            // Signing key must be first
-            let km = self.key_manager.lock().map_err(|_| Error::MutexPoisoned)?;
-            keys.insert(0, Basic::Ed25519.derive(km.public_key()));
-            let icp = EventMsgBuilder::new(EventType::Inception)
-                .with_prefix(&self.prefix)
-                .with_keys(keys)
-                .with_next_keys(vec!(Basic::Ed25519.derive(km.next_public_key())))
-                .build()?;
+    pub fn incept_with_extra_keys(
+        &mut self,
+        extra_keys: impl IntoIterator<Item = (Basic, PublicKey)>,
+    ) -> Result<SignedEventMessage, Error> {
+        let mut keys: Vec<BasicPrefix> = extra_keys
+            .into_iter()
+            .map(|(key_type, key)| key_type.derive(key))
+            .collect();
+        // Signing key must be first
+        let km = self.key_manager.lock().map_err(|_| Error::MutexPoisoned)?;
+        keys.insert(0, Basic::Ed25519.derive(km.public_key()?));
+        let icp = EventMsgBuilder::new(EventTypeTag::Icp)
+            .with_prefix(&self.prefix)
+            .with_keys(keys)
+            .with_next_keys(vec![Basic::Ed25519.derive(km.next_public_key()?)])
+            .build()?;
 
-            let signed = icp.sign(vec!(
-                AttachedSignaturePrefix::new(
-                    SelfSigning::Ed25519Sha512,
-                    km.sign(&icp.serialize()?)?,
-                    0
-                )
-            ), None);
-            self.processor.process(Message::Event(signed.clone()))?;
-            self.prefix = icp.event.prefix;
+        let signed = icp.sign(
+            vec![AttachedSignaturePrefix::new(
+                SelfSigning::Ed25519Sha512,
+                km.sign(&icp.serialize()?)?,
+                0,
+            )],
+            None,
+        );
+        self.processor
+            .process(Message::Event(Box::new(signed.clone())))?;
+        self.prefix = icp.event.get_prefix();
 
-            Ok(signed)
+        Ok(signed)
     }
 
     /// Interacts with peer identifier via generation of a `Seal`
@@ -171,7 +192,7 @@ impl<K: KeyManager> Keri<K> {
     pub fn interact(&self, peer: IdentifierPrefix) -> Result<SignedEventMessage, Error> {
         let next_sn = match self.processor.db.get_kel_finalized_events(&self.prefix) {
             Some(mut events) => match events.next_back() {
-                Some(db_event) => db_event.signed_event_message.event_message.event.sn + 1,
+                Some(db_event) => db_event.signed_event_message.event_message.event.get_sn() + 1,
                 None => return Err(Error::InvalidIdentifierStat),
             },
             None => return Err(Error::InvalidIdentifierStat),
@@ -186,14 +207,12 @@ impl<K: KeyManager> Keri<K> {
                 ))
             }
         };
-        let event = EventMessage::new(
-            Event::new(
-                self.prefix.clone(),
-                next_sn,
-                EventData::Ixn(InteractionEvent::new(pref, vec![seal])),
-            ),
-            SerializationFormats::JSON,
-        )?;
+        let event = Event::new(
+            self.prefix.clone(),
+            next_sn,
+            EventData::Ixn(InteractionEvent::new(pref, vec![seal])),
+        )
+        .to_message(SerializationFormats::JSON, &SelfAddressing::Blake3_256)?;
         let serialized = event.serialize()?;
         let signature = self
             .key_manager
@@ -205,8 +224,10 @@ impl<K: KeyManager> Keri<K> {
             signature,
             0, // TODO: what is this?
         );
-        let signed = SignedEventMessage::new(&event, vec!(asp), None);
-        self.processor.db.add_kel_finalized_event(signed.clone(), &self.prefix)?;
+        let signed = SignedEventMessage::new(&event, vec![asp], None);
+        self.processor
+            .db
+            .add_kel_finalized_event(signed.clone(), &self.prefix)?;
         Ok(signed)
     }
 
@@ -216,33 +237,36 @@ impl<K: KeyManager> Keri<K> {
             .map_err(|_| Error::MutexPoisoned)?
             .rotate()?;
         let rot = self.make_rotation()?;
-        let rot = rot.sign(vec![AttachedSignaturePrefix::new(
-            SelfSigning::Ed25519Sha512,
-            self.key_manager
+        let rot = rot.sign(
+            vec![AttachedSignaturePrefix::new(
+                SelfSigning::Ed25519Sha512,
+                self.key_manager
                     .lock()
                     .map_err(|_| Error::MutexPoisoned)?
                     .sign(&rot.serialize()?)?,
-            0,
-        )], None);
+                0,
+            )],
+            None,
+        );
 
         self.processor
-            .process(Message::Event(rot.clone()))?;
+            .process(Message::Event(Box::new(rot.clone())))?;
 
         Ok(rot)
     }
 
-    fn make_rotation(&self) -> Result<EventMessage, Error> {
+    fn make_rotation(&self) -> Result<EventMessage<KeyEvent>, Error> {
         let state = self
             .processor
             .compute_state(&self.prefix)?
-            .ok_or(Error::SemanticError("There is no state".into()))?;
+            .ok_or_else(|| Error::SemanticError("There is no state".into()))?;
         match self.key_manager.lock() {
-            Ok(kv) => EventMsgBuilder::new(EventType::Rotation)
+            Ok(kv) => EventMsgBuilder::new(EventTypeTag::Rot)
                 .with_prefix(&self.prefix)
                 .with_sn(state.sn + 1)
-                .with_previous_event(&SelfAddressing::Blake3_256.derive(&state.last))
-                .with_keys(vec![Basic::Ed25519.derive(kv.public_key())])
-                .with_next_keys(vec![Basic::Ed25519.derive(kv.next_public_key())])
+                .with_previous_event(&state.last_event_digest)
+                .with_keys(vec![Basic::Ed25519.derive(kv.public_key()?)])
+                .with_next_keys(vec![Basic::Ed25519.derive(kv.next_public_key()?)])
                 .build(),
             Err(_) => Err(Error::MutexPoisoned),
         }
@@ -260,26 +284,29 @@ impl<K: KeyManager> Keri<K> {
         let state = self
             .processor
             .compute_state(&self.prefix)?
-            .ok_or(Error::SemanticError("There is no state".into()))?;
+            .ok_or_else(|| Error::SemanticError("There is no state".into()))?;
 
-        let ev = EventMsgBuilder::new(EventType::Interaction)
+        let ev = EventMsgBuilder::new(EventTypeTag::Ixn)
             .with_prefix(&self.prefix)
             .with_sn(state.sn + 1)
-            .with_previous_event(&SelfAddressing::Blake3_256.derive(&state.last))
+            .with_previous_event(&state.last_event_digest)
             .with_seal(seal_list)
             .build()?;
 
-        let ixn = ev.sign(vec![AttachedSignaturePrefix::new(
-            SelfSigning::Ed25519Sha512,
-            self.key_manager
+        let ixn = ev.sign(
+            vec![AttachedSignaturePrefix::new(
+                SelfSigning::Ed25519Sha512,
+                self.key_manager
                     .lock()
                     .map_err(|_| Error::MutexPoisoned)?
                     .sign(&ev.serialize()?)?,
-            0,
-        )], None);
+                0,
+            )],
+            None,
+        );
 
         self.processor
-            .process(Message::Event(ixn.clone()))?;
+            .process(Message::Event(Box::new(ixn.clone())))?;
 
         Ok(ixn)
     }
@@ -290,12 +317,10 @@ impl<K: KeyManager> Keri<K> {
         let parsed = signed_message(msg).map_err(|e| Error::DeserializeError(e.to_string()))?;
         match Message::try_from(parsed.1) {
             Err(e) => Err(Error::DeserializeError(e.to_string())),
-            Ok(event) => {
-                match self.processor.process(event)? {
-                    None => Err(Error::InvalidIdentifierStat),
-                    Some(state) => Ok((state.prefix.clone(), serde_json::to_vec(&state)?)),
-                }
-            }
+            Ok(event) => match self.processor.process(event)? {
+                None => Err(Error::InvalidIdentifierStat),
+                Some(state) => Ok((state.prefix.clone(), serde_json::to_vec(&state)?)),
+            },
         }
     }
 
@@ -303,14 +328,15 @@ impl<K: KeyManager> Keri<K> {
         let events = signed_event_stream(msg)
             .map_err(|e| Error::DeserializeError(e.to_string()))?
             .1;
+
         let (processed_ok, _processed_failed): (Vec<_>, Vec<_>) = events
             .into_iter()
             .map(|event| {
-                self.processor
-                    .process(Message::try_from(event.clone()).unwrap())
-                    .and_then(|_| Message::try_from(event))
+                let message = Message::try_from(event)?;
+                self.processor.process(message.clone()).map(|_| message)
             })
             .partition(Result::is_ok);
+
         let response: Vec<u8> = processed_ok
             .into_iter()
             .map(Result::unwrap)
@@ -318,26 +344,26 @@ impl<K: KeyManager> Keri<K> {
                 match des_event {
                     Message::Event(ev) => {
                         let mut buf = vec![];
-                        if let EventData::Icp(_) = ev.event_message.event.event_data {
+                        if let EventData::Icp(_) = ev.event_message.event.get_event_data() {
                             if !self.processor.has_receipt(
                                 &self.prefix,
                                 0,
-                                &ev.event_message.event.prefix,
+                                &ev.event_message.event.get_prefix(),
                             )? {
                                 buf.append(
-                                    &mut self
-                                        .processor
-                                        .get_kerl(&self.prefix)?
-                                        .ok_or(Error::SemanticError("KEL is empty".into()))?,
+                                    &mut self.processor.get_kerl(&self.prefix)?.ok_or_else(
+                                        || Error::SemanticError("KEL is empty".into()),
+                                    )?,
                                 )
                             }
                         }
-                        let rcp: SignedEventData = self.make_rct(ev.event_message.clone())?.into();
+                        let rcp: SignedEventData = self.make_rct(ev.event_message)?.into();
                         buf.append(&mut rcp.to_cesr().unwrap());
                         Ok(buf)
                     }
+                    Message::TransferableRct(_rct) => Ok(vec![]),
                     // TODO: this should process properly
-                    _ => Ok(vec![]),
+                    _ => todo!(),
                 }
             })
             .filter_map(|x| x.ok())
@@ -346,7 +372,10 @@ impl<K: KeyManager> Keri<K> {
         Ok(response)
     }
 
-    pub fn make_rct(&self, event: EventMessage) -> Result<SignedTransferableReceipt, Error> {
+    pub fn make_rct(
+        &self,
+        event: EventMessage<KeyEvent>,
+    ) -> Result<SignedTransferableReceipt, Error> {
         let ser = event.serialize()?;
         let signature = self
             .key_manager
@@ -356,13 +385,11 @@ impl<K: KeyManager> Keri<K> {
         let validator_event_seal = self
             .processor
             .get_last_establishment_event_seal(&self.prefix)?
-            .ok_or(Error::SemanticError("No establishment event seal".into()))?;
-        let rcp = Event {
-            prefix: event.event.prefix,
-            sn: event.event.sn,
-            event_data: EventData::Rct(Receipt {
-                receipted_event_digest: SelfAddressing::Blake3_256.derive(&ser),
-            }),
+            .ok_or_else(|| Error::SemanticError("No establishment event seal".into()))?;
+        let rcp = Receipt {
+            prefix: event.event.get_prefix(),
+            sn: event.event.get_sn(),
+            receipted_event_digest: SelfAddressing::Blake3_256.derive(&ser),
         }
         .to_message(SerializationFormats::JSON)?;
 
@@ -371,10 +398,10 @@ impl<K: KeyManager> Keri<K> {
             signature,
             0,
         )];
-        let signed_rcp = SignedTransferableReceipt::new(&rcp, validator_event_seal, signatures);
+        let signed_rcp = SignedTransferableReceipt::new(rcp, validator_event_seal, signatures);
 
         self.processor
-            .process(Message::TransferableRct(signed_rcp.clone()))?;
+            .process(Message::TransferableRct(Box::new(signed_rcp.clone())))?;
 
         Ok(signed_rcp)
     }
@@ -387,7 +414,10 @@ impl<K: KeyManager> Keri<K> {
     /// # Parameters
     /// * `message` - `EventMessage` we are to process
     ///
-    pub fn make_ntr(&self, message: EventMessage) -> Result<SignedNontransferableReceipt, Error> {
+    pub fn make_ntr(
+        &self,
+        message: EventMessage<KeyEvent>,
+    ) -> Result<SignedNontransferableReceipt, Error> {
         let our_bp = match &self.prefix {
             IdentifierPrefix::Basic(prefix) => prefix,
             _ => {
@@ -396,17 +426,17 @@ impl<K: KeyManager> Keri<K> {
                 ))
             }
         };
-        match &message.event.event_data {
+        match &message.event.get_event_data() {
             // ICP requires check if we are in initial witnesses only
             EventData::Icp(evt) => {
-                if !evt.witness_config.initial_witnesses.contains(&our_bp) {
+                if !evt.witness_config.initial_witnesses.contains(our_bp) {
                     return Err(Error::SemanticError("we are not in a witness list.".into()));
                 }
                 self.generate_ntr(message)
             }
             EventData::Rot(evt) => {
-                if !evt.witness_config.prune.contains(&our_bp) {
-                    if evt.witness_config.graft.contains(&our_bp) {
+                if !evt.witness_config.prune.contains(our_bp) {
+                    if evt.witness_config.graft.contains(our_bp) {
                         // FIXME: logic for already witnessed identifier required
                         self.generate_ntr(message)
                     } else {
@@ -414,10 +444,10 @@ impl<K: KeyManager> Keri<K> {
                             "event does not change our status as a witness".into(),
                         ))
                     }
-                } else if evt.witness_config.prune.contains(&our_bp) {
+                } else if evt.witness_config.prune.contains(our_bp) {
                     self.processor
                         .db
-                        .remove_receipts_nt(&message.event.prefix)?;
+                        .remove_receipts_nt(&message.event.get_prefix())?;
                     Err(Error::SemanticError(
                         "we were removed. no receipt to generate".into(),
                     ))
@@ -452,29 +482,30 @@ impl<K: KeyManager> Keri<K> {
         self.processor.compute_state_at_sn(&seal.prefix, seal.sn)
     }
 
-    fn generate_ntr(&self, message: EventMessage) -> Result<SignedNontransferableReceipt, Error> {
+    fn generate_ntr(
+        &self,
+        message: EventMessage<KeyEvent>,
+    ) -> Result<SignedNontransferableReceipt, Error> {
         let signature;
         let bp;
         match self.key_manager.lock() {
             Ok(km) => {
                 signature = km.sign(&message.serialize()?)?;
-                bp = BasicPrefix::new(Basic::Ed25519, km.public_key());
+                bp = BasicPrefix::new(Basic::Ed25519, km.public_key()?);
             }
             Err(_) => return Err(Error::MutexPoisoned),
         }
         let ssp = SelfSigningPrefix::new(SelfSigning::Ed25519Sha512, signature);
-        let rcp = Event {
-            prefix: message.event.prefix.clone(),
-            sn: message.event.sn,
-            event_data: EventData::Rct(Receipt {
-                receipted_event_digest: SelfAddressing::Blake3_256.derive(&message.serialize()?),
-            }),
+        let rcp = Receipt {
+            prefix: message.event.get_prefix(),
+            sn: message.event.get_sn(),
+            receipted_event_digest: SelfAddressing::Blake3_256.derive(&message.serialize()?),
         }
         .to_message(SerializationFormats::JSON)?;
         let ntr = SignedNontransferableReceipt::new(&rcp, vec![(bp, ssp)]);
         self.processor
             .db
-            .add_receipt_nt(ntr.clone(), &message.event.prefix)?;
+            .add_receipt_nt(ntr.clone(), &message.event.get_prefix())?;
         Ok(ntr)
     }
 }
