@@ -1,25 +1,30 @@
 use crate::error::Error;
 use ed25519_dalek::{ExpandedSecretKey, SecretKey};
-use k256::ecdsa::{VerifyingKey, signature::{Verifier as EcdsaVerifier}};
-use k256::ecdsa::{Signature as EcdsaSignature, SigningKey, signature::{Signer as EcdsaSigner}};
+use k256::ecdsa::{signature::Signer as EcdsaSigner, Signature as EcdsaSignature, SigningKey};
+use k256::ecdsa::{signature::Verifier as EcdsaVerifier, VerifyingKey};
 use zeroize::Zeroize;
 
 #[derive(Debug, Clone, PartialEq, Hash)]
 pub struct PublicKey {
-    public_key: Vec<u8>
+    public_key: Vec<u8>,
 }
 
 impl PublicKey {
     pub fn new(key: Vec<u8>) -> Self {
-        PublicKey {public_key: key.to_vec()}
+        PublicKey { public_key: key }
     }
 
     pub fn key(&self) -> Vec<u8> {
         self.public_key.clone()
     }
 
+    /// Return true if the public key is empty.
+    pub fn is_empty(&self) -> bool {
+        self.public_key.is_empty()
+    }
+
     pub fn verify_ed(&self, msg: &[u8], sig: &[u8]) -> bool {
-        if let Ok(key) = ed25519_dalek::PublicKey::from_bytes(&self.key()) {
+        if let Ok(key) = ed25519_dalek::PublicKey::from_bytes(&self.public_key) {
             use arrayref::array_ref;
             if sig.len() != 64 {
                 return false;
@@ -35,7 +40,7 @@ impl PublicKey {
     }
 
     pub fn verify_ecdsa(&self, msg: &[u8], sig: &[u8]) -> bool {
-        match VerifyingKey::from_sec1_bytes(&self.key()) {
+        match VerifyingKey::from_sec1_bytes(&self.public_key) {
             Ok(k) => {
                 use k256::ecdsa::Signature;
                 use std::convert::TryFrom;
@@ -90,7 +95,7 @@ impl Drop for PrivateKey {
 
 #[test]
 fn libsodium_to_ed25519_dalek_compat() {
-    use ed25519_dalek::{Signature};
+    use ed25519_dalek::Signature;
     use rand::rngs::OsRng;
 
     let kp = ed25519_dalek::Keypair::generate(&mut OsRng);
@@ -105,7 +110,7 @@ fn libsodium_to_ed25519_dalek_compat() {
     assert!(sodium_pk.is_some());
     let sodium_pk = sodium_pk.unwrap();
     let mut sodium_sk_concat = kp.secret.to_bytes().to_vec();
-    sodium_sk_concat.append(&mut kp.public.to_bytes().to_vec().clone());
+    sodium_sk_concat.append(&mut kp.public.to_bytes().to_vec());
     let sodium_sk = sign::ed25519::SecretKey::from_slice(&sodium_sk_concat);
     assert!(sodium_sk.is_some());
     let sodium_sk = sodium_sk.unwrap();
@@ -113,7 +118,7 @@ fn libsodium_to_ed25519_dalek_compat() {
     let sodium_sig = sign::sign(msg, &sodium_sk);
 
     assert!(sign::verify_detached(
-        &sign::ed25519::Signature::new(dalek_sig.to_bytes()),
+        &sign::ed25519::Signature::from(dalek_sig.to_bytes()),
         msg,
         &sodium_pk
     ));
@@ -121,7 +126,7 @@ fn libsodium_to_ed25519_dalek_compat() {
     assert!(kp
         .verify(
             msg,
-            &Signature::new(arrayref::array_ref!(sodium_sig, 0, 64).to_owned())
+            &Signature::from(arrayref::array_ref!(sodium_sig, 0, 64).to_owned())
         )
         .is_ok());
 }
