@@ -3,10 +3,13 @@ pub mod derivation;
 pub mod error;
 pub mod event;
 pub mod event_message;
+pub mod event_parsing;
 pub mod keri;
+pub mod keys;
 pub mod prefix;
 pub mod processor;
 pub mod signer;
 pub mod state;
-pub mod keys;
-pub mod event_parsing;
+
+#[cfg(feature = "query")]
+pub mod query;
