@@ -1,10 +1,14 @@
 use super::{verify, Prefix, SelfSigningPrefix};
-use crate::{derivation::{basic::Basic, DerivationCode}, error::Error, keys::PublicKey};
+use crate::{
+    derivation::{basic::Basic, DerivationCode},
+    error::Error,
+    keys::PublicKey,
+};
 use base64::decode_config;
 use core::str::FromStr;
 use serde::{Deserialize, Deserializer, Serialize, Serializer};
 
-#[derive(Debug, Clone, Hash)]
+#[derive(Debug, Clone)]
 pub struct BasicPrefix {
     pub derivation: Basic,
     pub public_key: PublicKey,
@@ -102,9 +106,9 @@ fn serialize_deserialize() {
 
 #[test]
 fn to_from_string() {
+    use crate::keys::PrivateKey;
     use ed25519_dalek::Keypair;
     use rand::rngs::OsRng;
-    use crate::keys::PrivateKey;
 
     let kp = Keypair::generate(&mut OsRng);
 
