@@ -3,15 +3,14 @@ use crate::{
     keys::{PrivateKey, PublicKey},
 };
 use rand::rngs::OsRng;
-use std::ops::{Deref, DerefMut};
 
 #[cfg(feature = "wallet")]
 pub mod wallet;
 
 pub trait KeyManager {
     fn sign(&self, msg: &[u8]) -> Result<Vec<u8>, Error>;
-    fn public_key(&self) -> PublicKey;
-    fn next_public_key(&self) -> PublicKey;
+    fn public_key(&self) -> Result<PublicKey, Error>;
+    fn next_public_key(&self) -> Result<PublicKey, Error>;
     fn rotate(&mut self) -> Result<(), Error>;
 }
 
@@ -21,31 +20,17 @@ pub struct CryptoBox {
     pub next_pub_key: PublicKey,
 }
 
-impl DerefMut for CryptoBox {
-    fn deref_mut(&mut self) -> &mut Self::Target {
-        self
-    }
-}
-
-impl Deref for CryptoBox {
-    type Target = CryptoBox;
-
-    fn deref(&self) -> &Self::Target {
-        self
-    }
-}
-
 impl KeyManager for CryptoBox {
     fn sign(&self, msg: &[u8]) -> Result<Vec<u8>, Error> {
         self.signer.sign(msg)
     }
 
-    fn public_key(&self) -> PublicKey {
-        self.signer.pub_key.clone()
+    fn public_key(&self) -> Result<PublicKey, Error> {
+        Ok(self.signer.pub_key.clone())
     }
 
-    fn next_public_key(&self) -> PublicKey {
-        self.next_pub_key.clone()
+    fn next_public_key(&self) -> Result<PublicKey, Error> {
+        Ok(self.next_pub_key.clone())
     }
 
     fn rotate(&mut self) -> Result<(), Error> {
